﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using System.IO.IsolatedStorage;
using System.IO;

namespace IsoStorageSample
{
    public partial class Page : UserControl
    {

        // Name of folder to create 
        // in the isolated storage root.
        string subDirName = "AppFiles";

        // Set the maximum number of files
        // that can be added to the store  
        // (example of an application setting).
        int maxFilesNum = 10;

        ApplicationSettings appSettings = ApplicationSettings.Default;

        public Page()
        {
            InitializeComponent();

            // Add the setting if not already set.
            if (!appSettings.Contains("MaxFiles"))
            {
                appSettings.Add("MaxFiles", maxFilesNum);
            }

            try
            {
                using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                {

                    // Create a directory for the files
                    // if it doesn't already exist.
                    if (!store.DirectoryExists(subDirName))
                    {
                        store.CreateDirectory(subDirName);
                    }

                    // Determine existing files and add
                    // their names in the list box.
                    string[] filesInSubDir = store.GetFileNames(subDirName + "/*");
                    foreach (string fileName in filesInSubDir)
                    {
                        FileList.Items.Add(fileName);
                    }

                    // Clear alert text block
                    // and show store's statistics.
                    Alert.Text = "";

                    ShowStatistics(store);
                }
            }
            catch (IsolatedStorageException ex)
            {
                Alert.Text = ex.Message;
            }
        }

        // This method displays storage space usage information.
        private void ShowStatistics(IsolatedStorageFile store)
        {

            // Determine file count
            string[] curFiles = store.GetFileNames(subDirName + "/*");

            string spaceUsed = (store.Quota - store.AvailableFreeSpace).ToString();
            string spaceAvailable = store.AvailableFreeSpace.ToString();
            string curQuota = store.Quota.ToString();
            Status.Text =
                String.Format("Files: {0}\nQuota: {1} bytes\nAvail: {3} bytes\nUsed:  {2} bytes",
                    curFiles.Length.ToString(), curQuota, spaceUsed, spaceAvailable);

        }

        // This method creates a new file if the 
        // file name is not already in the list.
        private void OnClickSaveFile(object sender, RoutedEventArgs e)
        {
            // Get the application setting that
            // specifies the maximum number of files
            // that can be created in the isolated store.
            int curMax = (int)appSettings["MaxFiles"];


            // Do not continue if there is no file
            // name specified or if adding another
            // file would exceed the MaxFiles setting.
            if (FileNameBox.Text == "" || FileList.Items.Count + 1 > curMax)
            {
                if (FileNameBox.Text == "")
                {
                    Alert.Text = "File name required.";
                }
                else if (FileList.Items.Count + 1 > curMax)
                {
                    Alert.Text = "Cannot exceed " + curMax.ToString() + " files.";
                }
            }
            else
            {
                try
                {
                    using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                    {

                        // If a file name is selected in the list
                        // use that file name, otherwise use
                        // the name in the file name text box.
                        string fileName = "";
                        if (FileList.SelectedIndex != -1)
                        {
                            fileName = FileList.SelectedItem.ToString();
                        }
                        else
                        {
                            fileName = FileNameBox.Text;
                        }

                        try
                        {
                            // Create the file, overwriting it if it already exists. 
                            using (IsolatedStorageFileStream newFile =
                            store.CreateFile(System.IO.Path.Combine(subDirName, fileName)))
                            {
                                StreamWriter sw = new StreamWriter(newFile);
                                sw.Write(FileContentBox.Text);

                                // If a file is not selected in the list,
                                // use the file from the file name box.
                                if (FileList.SelectedIndex == -1)
                                {
                                    // Don't add the file name
                                    // if it is already in the list.
                                    bool isInList = false;
                                    for (int x = 0; x < FileList.Items.Count; x++)
                                    {
                                        if (FileList.Items[x].ToString().ToLower() ==
                                                        fileName.ToLower())
                                        {
                                            isInList = true;
                                            break;
                                        }

                                    }

                                    if (isInList == false)
                                    {

                                        FileList.Items.Add(fileName);
                                    }
                                }
                                sw.Close();

                                // Clear text boxes and display
                                // store usage information.
                                FileNameBox.Text = "";
                                FileContentBox.Text = "";
                                Alert.Text = "";
                                FileList.SelectedIndex = -1;
                                ShowStatistics(store);
                            }
                        }
                        catch (IsolatedStorageException ex)
                        {
                            Alert.Text = ex.Message;
                        }
                    }
                }
                catch (IsolatedStorageException ex)
                {
                    Alert.Text = ex.Message;
                }
            }
        }

        private void OnFileNameChange(object sender, RoutedEventArgs args)
        {

            // Remove any selection from the
            // file list box because a new 
            // file name is being created.
            FileList.SelectedIndex = -1;

        }

        // This method removes a file from isolated storage.
        private void OnClickDeleteFile(object sender, RoutedEventArgs e)
        {
            // Do not continue if a file is not selected.
            if (FileList.SelectedIndex != -1)
            {
                try
                {
                    using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                    {
                        string delFile = System.IO.Path.Combine(subDirName,
                            FileList.SelectedItem.ToString());

                        // Delete the file if it
                        // exists in the store.
                        if (store.FileExists(delFile))
                        {
                            try
                            {
                                store.DeleteFile(delFile);

                                // Remove the file name from the list.
                                FileList.Items.Remove(FileList.SelectedItem);

                                // Clear text boxes and display
                                // store usage information.
                                FileNameBox.Text = "";
                                FileContentBox.Text = "";
                                Alert.Text = "";
                                ShowStatistics(store);
                            }
                            catch (IsolatedStorageException ex)
                            {
                                Alert.Text = ex.Message;
                            }
                        }
                    }
                }
                catch (IsolatedStorageException ex)
                {

                    Alert.Text = ex.Message;
                }
            }
            else
            {
                Alert.Text = "Select a file to delete.";
            }
        }

        // This method displays the contents of selected file from the list.
        private void ShowContent(object sender, SelectionChangedEventArgs args)
        {
            // Do not continue if a file is not selected.
            if (FileList.SelectedIndex != -1)
            {
                try
                {
                    using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                    {
                        string fileName = FileList.SelectedItem.ToString();
                        FileNameBox.Text = fileName;

                        try
                        {
                            using (IsolatedStorageFileStream curFile =
                            store.OpenFile(System.IO.Path.Combine(subDirName, fileName),
                                    FileMode.Open, FileAccess.Read))
                            {

                                StreamReader sr = new StreamReader(curFile);
                                FileContentBox.Text = sr.ReadToEnd();
                                sr.Close();
                            }
                        }
                        catch (IsolatedStorageException ex)
                        {

                            Alert.Text = ex.Message;
                        }
                    }
                }
                catch (IsolatedStorageException ex)
                {
                    Alert.Text = ex.Message;
                }
            }
        }

        // This method starts the process to increase the
        // host's quota of storage space on the local computer. 
        private void OnClickIncreaseQuota(object sender, RoutedEventArgs args)
        {

            // Obtain an isolated store for an application.
            try
            {
                using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    // Request 5MB more space in bytes.
                    Int64 spaceToAdd = 5242880;
                    Int64 curAvail = store.AvailableFreeSpace;

                    // If available space is less than
                    // what is requested, try to increase.
                    if (curAvail < spaceToAdd)
                    {

                        // Request more quota space.
                        if (!store.TryIncreaseQuotaTo(store.Quota + spaceToAdd))
                        {
                            Alert.Text = "Increase rejected.";
                        }
                        else
                        {
                            Alert.Text = "Increase approved.";
                        }
                    }
                    else
                    {
                        Alert.Text = "5 MB is still available.";

                    }


                }
            }

            catch (IsolatedStorageException ex)
            {
                Alert.Text = ex.Message;

            }
        }
    }
}
